﻿using AutoMapper;
using CarShop.Data;
using CarShop.Logic;
using CarShop.Repository;
using CarShop.Web.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;

namespace CarShop.Web.Controllers
{
    public class CarsController : Controller
    {
        ICarLogic logic;
        IMapper mapper;
        CarsViewModel model;
        // asp.net core: transient + dep.injection
        // public CarsController(ICarLogic logic)

        // TODO: install EntityFramework...
        // TODO: copy connection string
        // TODO: copy database!!! CarShop.Web\App_Data\CarDatabase.mdf 
        // Manual DB copy/reset!
        // public CarsController()
        public CarsController(ICarLogic logic, IMapper mapper)
        {
            /*
            CarDbContext ctx = new CarDbContext();
            CarRepository carRepo = new CarRepository(ctx);
            BrandRepository brandRepo = new BrandRepository(ctx);
            logic = new CarLogic(carRepo, brandRepo);
            mapper = MapperFactory.CreateMapper();
            */
            // INSTEAD: Startup.cs, ConfigureServices();
            this.logic = logic;
            this.mapper = mapper;

            model = new CarsViewModel();
            model.EditedCar = new Models.Car();

            var cars = logic.GetAllCars();
            model.ListOfCars = mapper.Map<IList<Data.Car>, List<Models.Car>>(cars);
        }

        // GET: Cars
        public ActionResult Index()
        {
            ViewData["editAction"] = "AddNew";
            return View("CarsIndex", model);
        }

        private Models.Car GetCarModel(int id)
        {
            Data.Car oneCar = logic.GetOneCar(id);
            return mapper.Map<Data.Car, Models.Car>(oneCar);
        }

        // GET: Cars/Details/5
        public ActionResult Details(int id)
        {
            return View("CarsDetails", GetCarModel(id));
        }

        // NO separated create x2, edit x2, delete x2
        public ActionResult Remove(int id)
        {
            TempData["editResult"] = "Delete FAIL";
            if (logic.RemoveCar(id)) TempData["editResult"] = "Delete OK";
            return RedirectToAction(nameof(Index));
        }

        // GET Cars/Edit/20
        public ActionResult Edit(int id)
        {
            ViewData["editAction"] = "Edit";
            model.EditedCar = GetCarModel(id);
            return View("CarsIndex", model);
        }

        // POST Crud/Edit + send 1 car
        [HttpPost]
        public ActionResult Edit(Models.Car car, string editAction) 
            // Because of Model Binder
        {
            if (ModelState.IsValid && car != null)
            {
                TempData["editResult"] = "Edit OK";
                if (editAction == "AddNew")
                {
                    try
                    {
                        logic.AddCar(car.BrandName, car.Name, car.Price);
                    }
                    catch (ArgumentException ex)
                    {
                        TempData["editResult"] = "Insert FAIL: "+ex.Message;
                    }
                }
                else
                {
                    if (!logic.ChangeCar(car.Id, car.BrandName, car.Name, car.Price))
                    {
                        TempData["editResult"] = "Edit FAIL";
                    }
                }
                return RedirectToAction(nameof(Index));
            } else
            {
                ViewData["editAction"] = "Edit";
                model.EditedCar = car;
                return View("CarsIndex", model);
            }
        }
        // TODO: normal validation + exception/error handling
    }
}
